pub enum Trap {
Some(u64),
None,
}Expand description
Wether or not a trap occured after executing an instruction.
Variants
Some(u64)
The instruction trapped, contains the address of the trap handler.
None
The instruction did not trap.
Implementations